/********************************************************/
/*  ntapi: Native API core library                      */
/*  Copyright (C) 2013--2018  Z. Gilboa                 */
/*  Released under GPLv2 and GPLv3; see COPYING.NTAPI.  */
/********************************************************/

#include <psxtypes/psxtypes.h>
#include <ntapi/nt_tty.h>
#include <ntapi/ntapi.h>
#include "ntapi_impl.h"

#define __MSGBUF_ELEMENTS (NT_LPC_MAX_MSG_DATA_SIZE/sizeof(size_t)) + 1

int32_t	__stdcall __ntapi_tty_query_server_pts_slot_info(
	__in	void *			hport,
	__out	 nt_tty_pts_slot_info *	pts_slot_info,
	__in	intptr_t		blkidx)
{
	int32_t			status;
	nt_tty_server_msg *	msg;
	uintptr_t		msgbuf[__MSGBUF_ELEMENTS];

	__ntapi->tt_aligned_block_memset(
		msgbuf,0,
		sizeof(msgbuf));

	msg				= (nt_tty_server_msg *)msgbuf;
	msg->header.msg_type		= NT_LPC_NEW_MESSAGE;
	msg->header.data_size		= sizeof(msg->data);
	msg->header.msg_size		= sizeof(*msg);
	msg->data.ttyinfo.exarg         = (void *)blkidx;
	msg->data.ttyinfo.opcode	= NT_TTY_QUERY_INFORMATION_SERVER;
	msg->data.srvinfo.info_class	= NT_TTY_SERVER_PTS_SLOT_INFORMATION;

	if ((status = __ntapi->zw_request_wait_reply_port(hport,msg,msg)))
		return status;

	else if (msg->data.ttyinfo.status)
		return msg->data.ttyinfo.status;

	if (sizeof(*pts_slot_info) < msg->data.srvinfo.info_length)
		return NT_STATUS_INTERNAL_ERROR;

	__ntapi->tt_generic_memcpy(
		pts_slot_info,
		msg->data.srvinfo.info_buffer,
		msg->data.srvinfo.info_length);

	return NT_STATUS_SUCCESS;
}
